# -*- coding: utf-8 -*-
import logging
import time

from logging import StreamHandler

from django.conf import settings
from django.core.management.base import BaseCommand
from django.utils import timezone
from django.utils.translation import gettext_lazy as _

from ...models import Job, JobLog


class Command(BaseCommand):
    logger = logging.getLogger('cron')
    help = _('Delete Jobs')

    def add_arguments(self, parser):
        parser.add_argument(
            '-c', '--console', action='store_true', default=False,
            help='Debug - write logging to console',
        )
        parser.add_argument(
            '-l', '--debug-level', default='info',
            help='Set debug level (debug, info, warnings) for console',
        )
        parser.add_argument(
            '-d', '--cancellation-date', required=False,  # TODO help
            type=lambda d: timezone.datetime.strptime(d, '%Y-%m-%d'))

    def handle(self, *args, **options):
        debug_level = getattr(logging, options.get('debug_level').upper())
        self.logger.setLevel(debug_level)
        self.logger.handlers[0].setLevel(debug_level)

        if options.get('console'):
            console_handler = StreamHandler()
            console_handler.setLevel(debug_level)
            self.logger.addHandler(console_handler)
            self.logger.setLevel(debug_level)

        parameters = {}
        try:
            if options.get('cancellation-date', None):
                parameters['cancellation_date'] = options['cancellation-date']

            jobs = Job.selected_jobs_to_delete(**parameters)

            self.logger.info("Start delete Jobs")
            for num, job in enumerate(jobs):
                self.run(job)
            self.logger.info("End delete Jobs")

        except Exception as e:
            self.logger.error(f"Failed delete jobs: {e}")
            self.logger.exception(e)
            self.stdout.write(self.style.ERROR(
                f'Execution failed: {e}')
            )

    def run(self, job, restart_if_keyboard_interrupt: bool = False):
        self.logger.info("Start delete Job {} (id {})".format(job.name, job.id))
        try:
            result = job.delete_job()
            if result.is_deletion_valid:
                job.delete()
                self.logger.info("Delete Job {} (id {}) completed".format(job.name, job.id))
            else:
                self.logger.error("Delete Job {} error: {}".format(job.name, result.deletion_result))
        except Exception as e:
            self.logger.error("Delete Job {} error: {}".format(job.name, e))
            self.logger.exception(e)
            # raise
        except KeyboardInterrupt as e:
            self.logger.info("Job {}. Job deletion interrupted by user ...".format(job.name))
