# -*- coding: utf-8 -*-
import json
from django.contrib.auth import get_user_model
from django.utils.text import slugify
from django.utils.translation import gettext_lazy as _

from thx.notify.models import Notification

User = get_user_model()


class JobResultNotification:
    def __init__(self, job_class_instance, job_log_obj, *args, **kwargs):
        self.job_class_instance = job_class_instance
        self.job_log_obj = job_log_obj
        super().__init__(*args, **kwargs)

    def get_recipients(self):
        recipients_ids = self.job_log_obj.job.parameters.get("recipients_ids", False)
        if recipients_ids:
            recipients = User.objects.filter(id__in=recipients_ids)
        else:
            recipients = [self.job_log_obj.job.creator]
        return recipients

    def get_call_to_action_label(self):
        if hasattr(self.job_class_instance, "get_call_to_action_label"):
            return self.job_class_instance.get_call_to_action_label(self.job_log_obj)
        return None

    def call_to_action_url(self):
        if hasattr(self.job_class_instance, "get_call_to_action_url"):
            return self.job_class_instance.get_call_to_action_url(self.job_log_obj)
        return None

    def get_subject(self):
        if hasattr(self.job_class_instance, "get_notification_subject"):
            return self.job_class_instance.get_notification_subject(self.job_log_obj)
        return _("Job execution")

    def get_body(self):
        if hasattr(self.job_class_instance, "get_notification_body"):
            return self.job_class_instance.get_notification_body(self.job_log_obj)

        elaboration_results = self.job_log_obj.elaboration_results
        if isinstance(elaboration_results, str):
            elaboration_results = json.loads(elaboration_results)
        body = _(f"Job '{self.job_log_obj.job.name}' executed")
        if elaboration_results.get("success_count", None) and elaboration_results.get("error_count", None):
            success_count = elaboration_results['success_count']
            error_count = elaboration_results['error_count']
            body = _(f"{body}. Success count: {success_count}. Error count: {error_count}")
        return body

    def get_slug(self, subject=None):
        if hasattr(self.job_class_instance, "get_notification_slug"):
            return slugify(self.job_class_instance.get_notification_slug(self.job_log_obj))
        return slugify(str(subject))

    def make_job_notification(self, user):
        recipients = self.get_recipients()
        call_to_action_label = self.get_call_to_action_label()
        call_to_action_url = self.call_to_action_url()

        subject = self.get_subject()
        body = self.get_body()

        Notification.make_notification(
            recipients,
            subject,
            self.get_slug(subject),
            body,
            user,
            call_to_action_label,
            call_to_action_url,
        )
